module CrmJavaSearch
  module RelationDuplicate
    class RelationService
      attr_reader :lead_service, :contact_service, :customer_service, :organization, :lead_cross_fields,
                  :customer_cross_fields, :contact_cross_fields
      def initialize(organization)
        @organization = organization
        @lead_service = LeadService.new(organization)
        @contact_service = ContactService.new(organization)
        @customer_service = CustomerService.new(organization)
        @lead_cross_fields = nil
        @customer_cross_fields = nil
        @contact_cross_fields = nil
      end

      def lead_correlation
        {
          customer: duplicate_fields_intersections(lead_service.duplicate_fields,
                                                  customer_service.duplicate_fields),
          contact: duplicate_fields_intersections(lead_service.duplicate_fields,
                                                  contact_service.duplicate_fields)
        }
      end

      def contact_correlation
        {
          customer: duplicate_fields_intersections(contact_service.duplicate_fields,
                                                  customer_service.duplicate_fields),
          lead: duplicate_fields_intersections(contact_service.duplicate_fields,
                                              lead_service.duplicate_fields)
        }
      end

      def customer_correlation
        {
          lead: duplicate_fields_intersections(customer_service.duplicate_fields,
                                              lead_service.duplicate_fields),
          contact: duplicate_fields_intersections(customer_service.duplicate_fields,
                                                  contact_service.duplicate_fields)
        }
      end

      def lead_cross
        cross = ActiveSupport::HashWithIndifferentAccess.new
        if lead_cross_enable?
          cross_fields_data_prepare
          if cross_lead_customer?
            lead_cross_fields.each do |field|
              customer_field = if field[:name] == 'company_name'
                                customer_cross_fields.find { |c_field| c_field[:name] == 'name' }
                              else
                                customer_cross_fields.find { |c_field| c_field[:label] == field[:label] }
                              end
              cross[field[:name]] ||= {}
              cross[field[:name]][:customer] = customer_field[:name] if customer_field
            end
          end
          cross.deep_merge!(cross_fields_hash(lead_cross_fields, contact_cross_fields, :contact)) if
            cross_lead_contact?
          cross = unenable_cross_duplicate(lead_cross_fields) if !cross_lead_contact? && !cross_lead_customer?
        end
        Log.logger.info("lead_cross: #{cross}")
        cross
      end

      def customer_cross
        cross = HashWithIndifferentAccess.new
        if customer_cross_enable?
          cross_fields_data_prepare
          if cross_customer_lead?
            customer_cross_fields.each do |field|
              lead_field = case field[:name]
                          when 'name'
                            lead_cross_fields.find { |c_field| c_field[:name] == 'company_name' }
                          when 'company_name'
                            nil
                          else
                            lead_cross_fields.find { |c_field| c_field[:label] == field[:label] }
                          end
              cross[field[:name]] ||= {}
              cross[field[:name]][:lead] = lead_field[:name] if lead_field
            end
          end
          cross.deep_merge!(cross_fields_hash(customer_cross_fields, contact_cross_fields, :contact)) if
            cross_customer_contact?
          cross = unenable_cross_duplicate(customer_cross_fields) if !cross_customer_contact? &&
                                                                    !cross_customer_lead?
        end
        Log.logger.info("customer_cross: #{cross}")
        cross
      end

      def contact_cross
        cross = {}
        if contact_cross_enable?
          cross_fields_data_prepare
          cross =
            cross_fields_hash(contact_cross_fields, lead_cross_fields, :lead) if cross_contact_lead?
          cross.deep_merge!(cross_fields_hash(contact_cross_fields, customer_cross_fields, :customer)) if
            cross_contact_customer?
          cross = unenable_cross_duplicate(contact_cross_fields) if !cross_contact_customer? &&
                                                                    !cross_contact_lead?
        end
        Log.logger.info("contact_cross: #{cross}")
        cross
      end

      private

      def unenable_cross_duplicate(fields)
        cross = HashWithIndifferentAccess.new
        fields.each do |field|
          cross[field[:name]] = {}
        end
        cross
      end

      def cross_fields_hash(cross_fields1, cross_fields2, flag)
        cross = HashWithIndifferentAccess.new
        cross_fields1.each do |field|
          target_field = cross_fields2.find { |c_field| c_field[:label] == field[:label] }
          cross[field[:name]] ||= {}
          cross[field[:name]][flag] = target_field[:name] if target_field
        end
        cross
      end

      def cross_fields_data_prepare
        lead_service.cross = true
        contact_service.cross = true
        customer_service.cross = true
        @lead_cross_fields = lead_service.cross_fields
        @customer_cross_fields = customer_service.cross_fields
        @contact_cross_fields = contact_service.cross_fields
      end

      # 线索关联查重相关
      def lead_cross_enable?
        cross_setting.lead[:enabled].eql?('1')
      end

      def cross_lead_customer?
        cross_setting.lead[:customer_also].eql?('1')
      end

      def cross_lead_contact?
        cross_setting.lead[:contact_also].eql?('1')
      end

      # 客户关联查重相关
      def customer_cross_enable?
        cross_setting.customer[:enabled].eql?('1')
      end

      def cross_customer_lead?
        cross_setting.customer[:lead_also].eql?('1')
      end

      def cross_customer_contact?
        cross_setting.customer[:contact_also].eql?('1')
      end

      # 联系人关联查重相关
      def contact_cross_enable?
        cross_setting.contact[:enabled].eql?('1')
      end

      def cross_contact_lead?
        cross_setting.contact[:lead_also].eql?('1')
      end

      def cross_contact_customer?
        cross_setting.contact[:customer_also].eql?('1')
      end

      def cross_setting
        organization.settings(:check_duplicate_config)
      end

      def duplicate_fields_intersections(fields_set1, fields_set2)
        fields_set1.select do |field1|
          field1[:checked] && fields_set2.any?{|field2| field2[:checked] && field1[:field_type].eql?(field2[:field_type]) && field1[:label].eql?(field2[:label]) }
        end
      end
    end
  end
end
